#include <iostream>

#include <SmartDashboard/SmartDashboard.h>

#include <thirdParty/puxixml/pugixml.hpp>

#include <subsys/xmlParsing/MotorDefn.h>
#include <subsys/components/DragonTalon.h>

//-----------------------------------------------------------------------
// Method:      ParseXML
// Description: Parse a motor XML element and create a DragonTalon from
//              its definition.
// Returns:     DragonTalon*        motor controller (or nullptr if XML
//                                  is ill-formed
//-----------------------------------------------------------------------
DragonTalon* MotorDefn::ParseXML
(
    pugi::xml_node      motorNode
)
{
    DragonTalon* talon = nullptr;

    int canID = 0;
    DragonTalon::TALON_TYPE 	motorType = DragonTalon::UNKNOWN_TALON_TYPE;
    bool        				inverted = false;
    bool        				sensorInverted = false;
    CANTalon::FeedbackDevice 	feedbackDevice = CANTalon::QuadEncoder;
    int         				countsPerRev = 0;
    float       				gearRatio = 1;
    bool        				brakeMode = false;
    int         				currentLimit = 0;
    bool        				enableCurrentLimit = false;
    bool        				enableSafety = false;
    int         				masterID = -1;

    bool hasError = false;

    for (pugi::xml_attribute attr = motorNode.first_attribute(); attr; attr = attr.next_attribute())
    {
        if ( strcmp( attr.name(), "usage" ) == 0 )
        {
            motorType = attr.as_int();
        }
        else if ( strcmp( attr.name(), "canId" ) == 0 )
        {
            canID = attr.as_int();
        }
        else if ( strcmp( attr.name(), "type" ) == 0 )
        {
        	if ( strcmp( attr.value(), "cantalon" ) != 0 )
        	{
        		hasError = true;
        	}
        }
        else if ( strcmp( attr.name(), "inverted" ) == 0 )
        {
            inverted = attr.as_bool();
        }
        else if ( strcmp( attr.name(), "sensorInverted" ) == 0 )
        {
            sensorInverted = attr.as_bool();
        }
        else if ( strcmp( attr.name(), "feedbackDevice" ) == 0 )
        {
        	feedbackDevice = attr.as_int();
        }
        else if ( strcmp( attr.name(), "countsPerRev" ) == 0 )
        {
            countsPerRev = attr.as_int();
        }
        else if ( strcmp( attr.name(), "gearRatio" ) == 0 )
        {
            gearRatio = attr.as_float();
        }

        else if ( strcmp( attr.name(), "brakeMode" ) == 0 )
        {
            brakeMode = attr.as_bool();
        }
        else if ( strcmp( attr.name(), "currentLimiting" ) == 0 )
        {
            currentLimit = attr.as_int();
            if ( currentLimit > 0 )
            {
                enableCurrentLimit = true;
            }
        }
        else if ( strcmp( attr.name(), "safety" ) == 0 )
        {
            enableSafety = attr.as_bool();
        }
        else if ( strcmp( attr.name(), "masterID" ) == 0 )
        {
            masterID = attr.as_int();
        }
        else
        {
            printf( " invalid attribute &s \n", attr.name() );
            hasError = true;
        }
    }

    if ( !hasError )
    {
        printf( "Creating DragonTalon \n");
        talon = new DragonTalon( motorType, canID );
        talon->SetInverted( inverted );
        talon->SetSensorDirection( sensorInverted );
        talon->SetFeedbackDevice( feedbackDevice );
        printf( " sensor present %d \n", talon->IsSensorPresent( feedbackDevice ) );
        talon->ConfigEncoderCodesPerRev( countsPerRev * gearRatio );
        if ( brakeMode )
        {
            talon->ConfigNeutralMode( frc::CANSpeedController::kNeutralMode_Brake );
        }
        else
        {
            talon->ConfigNeutralMode( frc::CANSpeedController::kNeutralMode_Coast );
        }
        talon->SetCurrentLimit( currentLimit );
        talon->EnableCurrentLimit( enableCurrentLimit );
        talon->SetSafetyEnabled( enableSafety );
    }
    return talon;
}
