/*
 * IChassis.h
 *
 *  Created on: Jan 12, 2017
 *      Author: jonah shader
 */

#ifndef SRC_SUBSYS_ICHASSIS_H_
#define SRC_SUBSYS_ICHASSIS_H_

#include <CANTalon.h>

using namespace frc;

namespace Team302
{

	class IChassis
	{
		public:
			enum ChassisControlMode
			{
				VOLTAGE_MODE = CANTalon::TalonControlMode::kVoltageMode,
				POSITION_MODE = CANTalon::TalonControlMode::kPositionMode,
				VELOCITY_MODE = CANTalon::TalonControlMode::kSpeedMode,
				CURRENT_MODE = CANTalon::TalonControlMode::kCurrentMode,
				MAGIC_MOTION = CANTalon::TalonControlMode::kMotionMagicMode
			};

			//--------------------------------------------------------------------------------
			// Method:			IChassis
			// Description: 	Default Constructor
			// Returns: 		void
			//--------------------------------------------------------------------------------
			IChassis() = default;

			//--------------------------------------------------------------------------------
			// Method:			~IChassis
			// Description: 	Default Destructor
			// Returns: 		void
			//--------------------------------------------------------------------------------
			virtual ~IChassis() = default;

			//===========================  Accessors ========================================
			//==================== Velocity
			//--------------------------------------------------------------------------------
			// Method:			GetLeftFrontVelocity
			// Description: 	Return the calculated velocity of the left front wheel
			// Returns: 		float		velocity in inches / second
			//--------------------------------------------------------------------------------
			virtual float GetLeftFrontVelocity() const = 0;

			//--------------------------------------------------------------------------------
			// Method:			GetLeftBackVelocity
			// Description: 	Return the calculated velocity of the left Back wheel
			// Returns: 		float		velocity in inches / second
			//--------------------------------------------------------------------------------
			virtual float GetLeftBackVelocity() const = 0;

			//--------------------------------------------------------------------------------
			// Method:			GetRightFrontVelocity
			// Description: 	Return the calculated velocity of the right front wheel
			// Returns: 		float		velocity in inches / second
			//--------------------------------------------------------------------------------
			virtual float GetRightFrontVelocity() const = 0;

			//--------------------------------------------------------------------------------
			// Method:			GetRightBackVelocity
			// Description: 	Return the calculated velocity of the right Back wheel
			// Returns: 		float		velocity in inches / second
			//--------------------------------------------------------------------------------
			virtual float GetRightBackVelocity() const = 0;

			//==================== Displacement
			//--------------------------------------------------------------------------------
			// Method:			GetLeftFrontDistance
			// Description: 	Return the calculated distance of the left front wheel
			// Returns: 		float		velocity in inches
			//--------------------------------------------------------------------------------
			virtual float GetLeftFrontDistance() const = 0;

			//--------------------------------------------------------------------------------
			// Method:			GetLeftBackDistance
			// Description: 	Return the calculated distance of the left Back wheel
			// Returns: 		float		velocity in inches
			//--------------------------------------------------------------------------------
			virtual float GetLeftBackDistance() const = 0;

			//--------------------------------------------------------------------------------
			// Method:			GetRightFrontDistance
			// Description: 	Return the calculated distance of the right front wheel
			// Returns: 		float		velocity in inches
			//--------------------------------------------------------------------------------
			virtual float GetRightFrontDistance() const = 0;

			//--------------------------------------------------------------------------------
			// Method:			GetRightBackDistance
			// Description: 	Return the calculated distance of the right Back wheel
			// Returns: 		float		velocity in inches
			//--------------------------------------------------------------------------------
			virtual float GetRightBackDistance() const = 0;

			//===========================  Mutators ========================================
			//--------------------------------------------------------------------------------
			// Method:			ResetDistance
			// Description: 	Reset the distance accumulator, so that the GetxxxDistance calls
			//					are relative to the current position.
			// Returns: 		void
			//--------------------------------------------------------------------------------
			virtual void ResetDistance() = 0;

			//==================== Current Limiting
			//--------------------------------------------------------------------------------
			// Method:			SetCurrentLimit
			// Description: 	Set the maximum current for the drive wheels
			// Returns: 		void
			//--------------------------------------------------------------------------------
			virtual void SetCurrentLimit
			(
				int amps			// <I> - current limit in amps for each motor
			) = 0;

			//--------------------------------------------------------------------------------
			// Method:			EnableCurrentLimiting
			// Description: 	Turn on/off current limiting
			// Returns: 		int			error - 0 = everything ok
			//										1 = no current limit set
			//--------------------------------------------------------------------------------
			virtual int EnableCurrentLimiting
			(
				bool	on			// <I> - true - turn on current enabling, false - turn it off
			) = 0;


			//==================== Neutral Mode
			//--------------------------------------------------------------------------------
			// Method:			SetNeutralMode
			// Description: 	Set the neutral mode on the motor
			// Returns: 		void
			//--------------------------------------------------------------------------------
			virtual void SetNeutralMode
			(
				CANSpeedController::NeutralMode		mode //	 <I> - the CANSpeedController neutral mode (usually either kNeutralMode_Brake or kNeutralMode_Coast)
			) = 0;

			// TODO:  Change the interface to force the implementation to decide the power to each wheel
			// TODO:  Should probably be velocity and heading based with an orientation option.
			//==================== Open Loop
			//--------------------------------------------------------------------------------
			// Method:			SetOpenLoopPower
			// Description: 	Set the power to the drive wheels (this is using a talon
			//					control mode kThrottleMode).
			//
			//					Values outside the range will be snapped to closest bounding value.
			//
			//					Due to different chassis configurations (e.g. some chassis may have
			//					wheels mechanically linked together).  Some motors will be followers
			//					or slaves to the masters.  In these cases the power to the from
			//					motors will be used and the back motors will be ignored.
			//
			// Returns: 		void
			//--------------------------------------------------------------------------------
			virtual void SetOpenLoopPower
			(
				float	leftFrontPower,			// <I> - left front motor power ( -1.0 < value < 1.0 )
				float	leftBackPower,			// <I> - left back motor power ( -1.0 < value < 1.0 )
				float	rightFrontPower,		// <I> - right front motor power ( -1.0 < value < 1.0 )
				float	rightBackPower			// <I> - right back motor power ( -1.0 < value < 1.0 )
			) = 0;

			// TODO:: Should closed loop read the constants from the config file or are they passed in?
			// TODO:  If passed in should it be an object that provides these values instead of passing
			// TODO:  so many items?
			//==================== Closed Loop
			//--------------------------------------------------------------------------------
			// Method:			SetClosedLoopParameters
			// Description: 	Set the power to the drive wheels.  Values outside the range
			//					will be snapped to closest bounding value.
			// Returns: 		void
			//--------------------------------------------------------------------------------
			virtual void SetClosedLoopParameters
			(
				ChassisControlMode	mode,	// <I> - talon control mode
				float	proportionalConst,	// <I> - kP from a PIDF
				float	integralConst,		// <I> - kI from a PIDF
				float	derivativeConst,	// <I> - kD from a PIDF
				float	feetForwardConst,	// <I> - kF from a PIDF
				float	leftFrontTarget,	// <I> - left front motor target (units are based on the mode where distances are inches and time is in seconds)
				float	leftBackTarget,		// <I> - left back motor power (units are based on the mode where distances are inches and time is in seconds)
				float	rightFrontTarget,	// <I> - right front motor power (units are based on the mode where distances are inches and time is in seconds)
				float	rightBackTarget		// <I> - right back motor power (units are based on the mode where distances are inches and time is in seconds)
			) = 0;
	};

} /* namespace Team302 */

#endif /* SRC_SUBSYS_ICHASSIS_H_ */
