/*
 * MecanumChassis.cpp
 *
 */

#include <math.h>
#include <CANTalon.h>
#include <subsys/IChassis.h>
#include <subsys/MecanumChassis.h>
#include <subsys/RobotMap.h>
#include <SmartDashboard/SmartDashboard.h>
#include <utils/LimitValue.h>

namespace Team302
{

	MecanumChassis::MecanumChassis()  : IChassis(),
										m_driveMotors(),
										m_amps( -1.0 )
	{
		RobotMap* map = RobotMap::GetInstance();
		if ( map != nullptr )
		{
			for ( int inx=RobotMap::DRIVE_MOTORS::LEFT_FRONT; inx<RobotMap::DRIVE_MOTORS::MAX_DRIVE_MOTORS; ++inx )
			{
				RobotMap::DRIVE_MOTORS motor = (RobotMap::DRIVE_MOTORS) inx;
				// Create Motor Controller
				m_driveMotors[ inx ] = new CANTalon( map->GetDriveMotorCANID( motor ) );

				// Motor Control Settings
				m_driveMotors[ inx ]->SetInverted( map->IsDriveMotorInverted( motor ) );

				// Feedback Device Settings
				CANTalon::FeedbackDevice feedback = map->GetDriveMotorFeedbackDevice( motor );
				m_driveMotors[ inx ]->SetFeedbackDevice( feedback );
				switch ( feedback )
				{
					case CANTalon::FeedbackDevice::QuadEncoder:
					case CANTalon::FeedbackDevice::EncFalling:
					case CANTalon::FeedbackDevice::EncRising:
						m_driveMotors[ inx ]->ConfigEncoderCodesPerRev( map->GetCountsPerRev( motor ) );
						break;

					case CANTalon::FeedbackDevice::AnalogPot:
					case CANTalon::FeedbackDevice::AnalogEncoder:
						m_driveMotors[ inx ]->ConfigPotentiometerTurns( map->GetTurnsPerRev( motor ) );
						break;

					default:
						break;
				}
				m_driveMotors[ inx ]->SetSensorDirection( map->IsSensorInverted( motor ) );

				m_driveMotors[ inx ]->IsSafetyEnabled();
			}
		}

	}
	//===========================  Accessors ========================================
	//==================== Velocity
	//--------------------------------------------------------------------------------
	// Method:			GetLeftFrontVelocity
	// Description: 	Return the calculated velocity of the left front wheel
	// Returns: 		float		velocity in inches / second
	//--------------------------------------------------------------------------------
	float MecanumChassis::GetLeftFrontVelocity() const
	{
		float velocity = m_driveMotors[ RobotMap::DRIVE_MOTORS::LEFT_FRONT ]->GetEncVel();
		return velocity;
	}

	//--------------------------------------------------------------------------------
	// Method:			GetLeftBackVelocity
	// Description: 	Return the calculated velocity of the left Back wheel
	// Returns: 		float		velocity in inches / second
	//--------------------------------------------------------------------------------
	float MecanumChassis::GetLeftBackVelocity() const
	{
		float velocity = m_driveMotors[ RobotMap::DRIVE_MOTORS::LEFT_BACK ]->GetEncVel();
		return velocity;
	};

	//--------------------------------------------------------------------------------
	// Method:			GetRightFrontVelocity
	// Description: 	Return the calculated velocity of the right front wheel
	// Returns: 		float		velocity in inches / second
	//--------------------------------------------------------------------------------
	float MecanumChassis::GetRightFrontVelocity() const
	{
		float velocity = m_driveMotors[ RobotMap::DRIVE_MOTORS::RIGHT_FRONT ]->GetEncVel();
		return velocity;
	}

	//--------------------------------------------------------------------------------
	// Method:			GetRightBackVelocity
	// Description: 	Return the calculated velocity of the right Back wheel
	// Returns: 		float		velocity in inches / second
	//--------------------------------------------------------------------------------
	float MecanumChassis::GetRightBackVelocity() const
	{
		float velocity = m_driveMotors[ RobotMap::DRIVE_MOTORS::RIGHT_BACK ]->GetEncVel();
		return velocity;
	}

	//==================== Displacement
	//--------------------------------------------------------------------------------
	// Method:			GetLeftFrontDistance
	// Description: 	Return the calculated distance of the left front wheel
	// Returns: 		float		distance in inches
	//--------------------------------------------------------------------------------
	float MecanumChassis::GetLeftFrontDistance() const
	{
		float distance = 0.0;						// output value
		RobotMap* map = RobotMap::GetInstance();	// Get the map

		// Read the counts and multiply it by the inch conversion factor
		auto count = m_driveMotors[ RobotMap::DRIVE_MOTORS::LEFT_FRONT ]->GetEncPosition();
		auto factor = map->GetCountsToInchConversionFactor( RobotMap::DRIVE_MOTORS::LEFT_FRONT );
		distance = count * factor;
		return distance;
	}

	//--------------------------------------------------------------------------------
	// Method:			GetLeftBackDistance
	// Description: 	Return the calculated distance of the left Back wheel
	// Returns: 		float		distance in inches
	//--------------------------------------------------------------------------------
	float MecanumChassis::GetLeftBackDistance() const
	{
		float distance = 0.0;  // TODO: read encoder
		RobotMap* map = RobotMap::GetInstance();	// Get the map

		// Read the counts and multiply it by the inch conversion factor
		auto count = m_driveMotors[ RobotMap::DRIVE_MOTORS::LEFT_BACK ]->GetEncPosition();
		auto factor = map->GetCountsToInchConversionFactor( RobotMap::DRIVE_MOTORS::LEFT_BACK );
		distance = count * factor;
		return distance;
	}

	//--------------------------------------------------------------------------------
	// Method:			GetRightFrontDistance
	// Description: 	Return the calculated distance of the right front wheel
	// Returns: 		float		distance in inches
	//--------------------------------------------------------------------------------
	float MecanumChassis::GetRightFrontDistance() const
	{
		float distance = 0.0;  // TODO: read encoder
		RobotMap* map = RobotMap::GetInstance();	// Get the map

		// Read the counts and multiply it by the inch conversion factor
		auto count = m_driveMotors[ RobotMap::DRIVE_MOTORS::RIGHT_FRONT ]->GetEncPosition();
		auto factor = map->GetCountsToInchConversionFactor( RobotMap::DRIVE_MOTORS::RIGHT_FRONT );
		distance = count * factor;
		return distance;
	}

	//--------------------------------------------------------------------------------
	// Method:			GetRightBackDistance
	// Description: 	Return the calculated distance of the right Back wheel
	// Returns: 		float		distance in inches
	//--------------------------------------------------------------------------------
	float MecanumChassis::GetRightBackDistance() const
	{
		float distance = 0.0;  // TODO: read encoder
		RobotMap* map = RobotMap::GetInstance();	// Get the map

		// Read the counts and multiply it by the inch conversion factor
		auto count = m_driveMotors[ RobotMap::DRIVE_MOTORS::RIGHT_BACK ]->GetEncPosition();
		auto factor = map->GetCountsToInchConversionFactor( RobotMap::DRIVE_MOTORS::RIGHT_BACK );
		distance = count * factor;
		return distance;
	}

	//===========================  Mutators ========================================
	//--------------------------------------------------------------------------------
	// Method:			ResetDistance
	// Description: 	Reset the distance accumulator, so that the GetxxxDistance calls
	//					are relative to the current position.
	// Returns: 		void
	//--------------------------------------------------------------------------------
	void MecanumChassis::ResetDistance()
	{
		// Reset the position for all drive motors
		for ( int inx=RobotMap::DRIVE_MOTORS::LEFT_FRONT; inx<RobotMap::DRIVE_MOTORS::MAX_DRIVE_MOTORS; ++inx )
		{
			m_driveMotors[ inx ]->SetEncPosition( 0 );
		}
	}

	//==================== Current Limiting
	//--------------------------------------------------------------------------------
	// Method:			SetCurrentLimit
	// Description: 	Set the maximum current for the drive wheels
	// Returns: 		void
	//--------------------------------------------------------------------------------
	void MecanumChassis::SetCurrentLimit
	(
		int amps			// <I> - current limit in amps for each motor
	)
	{
		if ( amps > 0.0 )
		{
			for ( int inx=RobotMap::DRIVE_MOTORS::LEFT_FRONT; inx<RobotMap::DRIVE_MOTORS::MAX_DRIVE_MOTORS; ++inx )
			{
				m_driveMotors[ inx ]->SetCurrentLimit( amps );
			}
		}
	}
	//--------------------------------------------------------------------------------
	// Method:			EnableCurrentLimiting
	// Description: 	Turn on/off current limiting
	// Returns: 		int			error - 0 = everything ok
	//										1 = no current limit set
	//--------------------------------------------------------------------------------
	int MecanumChassis::EnableCurrentLimiting
	(
		bool	on			// <I> - true - turn on current enabling, false - turn it off
	)
	{
		int error = 0;
		if ( m_amps > 0.0 )
		{
			for ( int inx=RobotMap::DRIVE_MOTORS::LEFT_FRONT; inx<RobotMap::DRIVE_MOTORS::MAX_DRIVE_MOTORS; ++inx )
			{
				m_driveMotors[ inx ]->EnableCurrentLimit( on );
			}
		}
		else
		{
			printf( "==>> ERROR: amp limit needs to be set MecanumChassis::EnableCurrentLimiting \n " );
			error = 1;
		}
		return error;
	}


	//==================== Neutral Mode
	//--------------------------------------------------------------------------------
	// Method:			SetNeutralMode
	// Description: 	Set the neutral mode on the motor
	// Returns: 		void
	//--------------------------------------------------------------------------------
	void MecanumChassis::SetNeutralMode
	(
		CANSpeedController::NeutralMode		mode //	 <I> - the CANSpeedController neutral mode (usually either kNeutralMode_Brake or kNeutralMode_Coast)
	)
	{
		for ( int inx=RobotMap::DRIVE_MOTORS::LEFT_FRONT; inx<RobotMap::DRIVE_MOTORS::MAX_DRIVE_MOTORS; ++inx )
		{
			m_driveMotors[ inx ]->ConfigNeutralMode( mode );
		}
	}

	//==================== Open Loop
	//--------------------------------------------------------------------------------
	// Method:			SetOpenLoopPower
	// Description: 	Set the power to the drive wheels (this is using a talon
	//					control mode kThrottleMode).
	//
	//					Values outside the range will be snapped to closest bounding value.
	//
	//					Due to different chassis configurations (e.g. some chassis may have
	//					wheels mechanically linked together).  Some motors will be followers
	//					or slaves to the masters.  In these cases the power to the from
	//					motors will be used and the back motors will be ignored.
	//
	// Returns: 		void
	//--------------------------------------------------------------------------------
	void MecanumChassis::SetOpenLoopPower
	(
		float	leftFrontPower,			// <I> - left front motor power ( -1.0 < value < 1.0 )
		float	leftBackPower,			// <I> - left back motor power ( -1.0 < value < 1.0 )
		float	rightFrontPower,		// <I> - right front motor power ( -1.0 < value < 1.0 )
		float	rightBackPower 			// <I> - right back motor power ( -1.0 < value < 1.0 )
	)
	{
		for ( int inx=RobotMap::DRIVE_MOTORS::LEFT_FRONT; inx<RobotMap::DRIVE_MOTORS::MAX_DRIVE_MOTORS; ++inx )
		{
			m_driveMotors[ RobotMap::DRIVE_MOTORS::LEFT_FRONT ] ->Set( LimitValue::ForceInRange( leftFrontPower, -0.98, 0.98 ) );
			m_driveMotors[ RobotMap::DRIVE_MOTORS::LEFT_BACK ]  ->Set( LimitValue::ForceInRange( leftBackPower, -0.98, 0.98 ) );
			m_driveMotors[ RobotMap::DRIVE_MOTORS::RIGHT_FRONT ]->Set( LimitValue::ForceInRange( rightFrontPower, -0.98, 0.98 ) );
			m_driveMotors[ RobotMap::DRIVE_MOTORS::RIGHT_BACK ] ->Set( LimitValue::ForceInRange( rightBackPower, -0.98, 0.98 ) );
		}
	}

	//==================== Closed Loop
	//--------------------------------------------------------------------------------
	// Method:			SetClosedLoopParameters
	// Description: 	Set the power to the drive wheels.  Values outside the range
	//					will be snapped to closest bounding value.
	// Returns: 		void
	//--------------------------------------------------------------------------------
	void MecanumChassis::SetClosedLoopParameters
	(
		ChassisControlMode	mode,	// <I> - talon control mode
		float	proportionalConst,	// <I> - kP from a PIDF
		float	integralConst,		// <I> - kI from a PIDF
		float	derivativeConst,	// <I> - kD from a PIDF
		float	feetForwardConst,	// <I> - kF from a PIDF
		float	leftFrontTarget,	// <I> - left front motor target (units are based on the mode where distances are inches and time is in seconds)
		float	leftBackTarget,		// <I> - left back motor power (units are based on the mode where distances are inches and time is in seconds)
		float	rightFrontTarget,	// <I> - right front motor power (units are based on the mode where distances are inches and time is in seconds)
		float	rightBackTarget		// <I> - right back motor power (units are based on the mode where distances are inches and time is in seconds)
	)
	{
		// TODO: need to write
	}

} /* namespace Team302 */
