/*
 * RobotMap.h
 *
 *  Created on: Jan 14, 2017
 *      Author: Team302
 */

#ifndef ROBOTMAP_H_
#define ROBOTMAP_H_

#include <CANTalon.h>

namespace Team302
{
	class RobotMap
	{
		public:
			//----------------------------------------------------------------------------------
			// Method:      GetInstance
			// Description: If there isn't an instance of the RobotMap, create it.
			//              Otherwise, hand out the single instance.
			// Returns:     RobotMap*   RobotMap that contains the physical characteristics of
			//							the robot
			//----------------------------------------------------------------------------------
			static RobotMap* GetInstance();

			enum DRIVE_MOTORS
			{
				LEFT_FRONT,
				LEFT_BACK,
				RIGHT_FRONT,
				RIGHT_BACK,
				MAX_DRIVE_MOTORS
			};


			//--------------------------------------------------------------------------------
			// 	Method:  		GetDriveMotorCANID
			//	Description:	Accessor for the Drive Motor CAN IDs
			//	Returns:		int		- CAN ID
			//--------------------------------------------------------------------------------
			int GetDriveMotorCANID
			(
				RobotMap::DRIVE_MOTORS	motor	// <I> - Motor to query
			) const;

			//--------------------------------------------------------------------------------
			// 	Method:  		IsDriveMotorInverted
			//	Description:	Accessor for the whether the Drive Motor is inverted or not
			//	Returns:		bool		- true	= motor inverted
			//								  false = motor is not inverted
			//--------------------------------------------------------------------------------
			bool IsDriveMotorInverted
			(
				RobotMap::DRIVE_MOTORS	motor	// <I> - Motor to query
			) const;

			//--------------------------------------------------------------------------------
			// 	Method:  		GetDriveMotorFeedbackDevice
			//	Description:	Accessor for the Drive Motor feedback device
			//	Returns:		CANTalon::FeedbackDevice	feedback device type
			//--------------------------------------------------------------------------------
			CANTalon::FeedbackDevice GetDriveMotorFeedbackDevice
			(
				RobotMap::DRIVE_MOTORS	motor	// <I> - Motor to query
			) const;

			//--------------------------------------------------------------------------------
			// 	Method:  		IsSensorInverted
			//	Description:	Accessor for the whether the Drive Motor's sensor is inverted or not
			//	Returns:		bool		- true	= sensor is inverted
			//								  false = sensor is not inverted
			//--------------------------------------------------------------------------------
			bool IsSensorInverted
			(
				RobotMap::DRIVE_MOTORS	motor	// <I> - Motor to query
			) const;

			//--------------------------------------------------------------------------------
			// 	Method:  		GetCountsPerRev
			//	Description:	Accessor for the Drive Motor's encoder counts per revolution
			//	Returns:		uint16_t	number of counts per revolution
			//--------------------------------------------------------------------------------
			uint16_t GetCountsPerRev
			(
				RobotMap::DRIVE_MOTORS	motor	// <I> - Motor to query
			) const;

			//--------------------------------------------------------------------------------
			// 	Method:  		GetTurnsPerRev
			//	Description:	Accessor for the Drive Motor's analog sensor turns per revolution
			//	Returns:		uint16_t	number of turns per revolution
			//--------------------------------------------------------------------------------
			uint16_t GetTurnsPerRev
			(
				RobotMap::DRIVE_MOTORS	motor	// <I> - Motor to query
			) const;

			//--------------------------------------------------------------------------------
			// 	Method:  		GetGearRatio
			//	Description:	Accessor for the Drive Motor's gear ratio
			//	Returns:		float	number of of the wheel shaft compared to the number of
			//							turns of the motor shaft if the sensor is mounted on the
			//							motor shaft.
			//--------------------------------------------------------------------------------
			float GetGearRatio
			(
				RobotMap::DRIVE_MOTORS	motor	// <I> - Motor to query
			) const;

			//--------------------------------------------------------------------------------
			// 	Method:  		IsSafetyEnabled
			//	Description:	Accessor for whether safety is on or off
			//	Returns:		bool  	true  = safety enabled
			//							false = safety disabled
			//--------------------------------------------------------------------------------
			inline bool IsSafetyEnabled() const { return m_safetyEnabled; }


			//--------------------------------------------------------------------------------
			// 	Method:  		GetWheelDiameter
			//	Description:	Accessor for the Drive Motor's wheel diameter
			//	Returns:		float		diameter in inches
			//--------------------------------------------------------------------------------
			float GetWheelDiameter
			(
				RobotMap::DRIVE_MOTORS	motor	// <I> - Motor to query
			) const;

			//--------------------------------------------------------------------------------
			// 	Method:  		GetCountsToInchConversionFactor
			//	Description:	Accessor for the Drive Motor's analog sensor turns per revolution
			//	Returns:		uint16_t	number of turns per revolution
			//--------------------------------------------------------------------------------
			float GetCountsToInchConversionFactor
			(
				RobotMap::DRIVE_MOTORS	motor	// <I> - Motor to query
			) const;

		private:

			RobotMap();
			~RobotMap() = default;


			static RobotMap*	m_instance;

			int   m_leftFrontDriveMotor;
			int   m_leftBackDriveMotor;
			int   m_rightFrontDriveMotor;
			int   m_rightBackDriveMotor;

			bool  m_leftFrontDriveMotorInverted;
			bool  m_leftBackDriveMotorInverted;
			bool  m_rightFrontDriveMotorInverted;
			bool  m_rightBackDriveMotorInverted;

			bool  m_leftFrontDriveSensorInverted;
			bool  m_leftBackDriveSensorInverted;
			bool  m_rightFrontDriveSensorInverted;
			bool  m_rightBackDriveSensorInverted;

			CANTalon::FeedbackDevice  m_leftFrontDriveMotorFeedbackType;
			CANTalon::FeedbackDevice  m_leftBackDriveMotorFeedbackType;
			CANTalon::FeedbackDevice  m_rightFrontDriveMotorFeedbackType;
			CANTalon::FeedbackDevice  m_rightBackDriveMotorFeedbackType;

			uint16_t m_leftFrontCountsPerRev;
			uint16_t m_leftBackCountsPerRev;
			uint16_t m_rightFrontCountsPerRev;
			uint16_t m_rightBackCountsPerRev;

			uint16_t m_leftFrontTurnsPerRev;
			uint16_t m_leftBackTurnsPerRev;
			uint16_t m_rightFrontTurnsPerRev;
			uint16_t m_rightBackTurnsPerRev;

			float m_leftFrontGearRatio;
			float m_leftBackGearRatio;
			float m_rightFrontGearRatio;
			float m_rightBackGearRatio;

			bool  m_safetyEnabled;


			float m_leftFrontWheelDiameter;
			float m_leftBackWheelDiameter;
			float m_rightFrontWheelDiameter;
			float m_rightBackWheelDiameter;

			float m_leftFrontWheelCountToInchConversion;
			float m_leftBackWheelCountToInchConversion;
			float m_rightFrontWheelCountToInchConversion;
			float m_rightBackWheelCountToInchConversion;


			//============================================================================
			// CAN PORTS
			//============================================================================

			// Driving
			const int LEFT_FRONT_DRIVE_MOTOR			= 1;
			const int LEFT_BACK_DRIVE_MOTOR				= 2;
			const int RIGHT_FRONT_DRIVE_MOTOR			= 3; //invert
			const int RIGHT_BACK_DRIVE_MOTOR			= 4; //invert


			//============================================================================
			// TALON Motor- inversion settings
			//============================================================================
			//TODO: in compBotChassis, we are currently only using 1 encoder for distance. we need to check all encoders and use as much as possible
			const bool IS_LEFT_FRONT_DRIVE_MOTOR_INVERTED	   	    = false;
			const bool IS_LEFT_BACK_DRIVE_MOTOR_INVERTED		    = false;
			const bool IS_RIGHT_FRONT_DRIVE_MOTOR_INVERTED			= true;
			const bool IS_RIGHT_BACK_DRIVE_MOTOR_INVERTED		    = true;

			//============================================================================
			// TALON Encoder- sensor inversion settings
			//============================================================================

			const bool IS_LEFT_FRONT_DRIVE_ENCODER_INVERTED		    = true;
			const bool IS_LEFT_BACK_DRIVE_ENCODER_INVERTED		    = true;
			const bool IS_RIGHT_FRONT_DRIVE_ENCODER_INVERTED	    = true;
			const bool IS_RIGHT_BACK_DRIVE_ENCODER_INVERTED		    = true;

			//==========================================================================================
			// Drive Encoder conversion rates to convert from counts to feet and feet per second
			//==========================================================================================
		//	const float ENCODER_DISTANCE_CONVERSION = 0.0023271056693257; // encoder counts to feet //Comp Bot
			const float ENCODER_DISTANCE_CONVERSION = 0.0019023912023129; //comp bot 4-12
		//	const float ENCODER_DISTANCE_CONVERSION = 0.0024375380865326; //Eighty Bot
			const float ENCODER_VELOCITY_CONVERSION = 0.0232710566932577; // encoder counts per 0.1 sec to feet per second
			const float DEGREES_PER_PULSE = (90.0/(1150.0 * 1.0)); //(90.0/(1350.0 * 1.0)) (90.0/(1323.0 * 4.0))  90 degrees divided by delta counts (left - right) from calibrated turn times for to convert to quad encoding. degrees/(counts * encoding)
			//TODO: fix this. remove the ratio at the end of the conversions and figure out the correct conversion
			const float FPS_TO_RPM = 38.197097020626; //conversion from feet per second to rpm for use with talon velocity PID control //TODO: need correct value here
			const float FEET_TO_ROTATIONS = 1.5707963267948966; //conversion from feet to rotations for use witl talon position PID control

			const float DRIVE_COUNTS_PER_REVOLUTION = 450; //encoder counts per rotation of drive motors //360

			//=============================================================================================
			// Digital IO
			//=============================================================================================
			const int GYRO_SENSOR									= 1;
	};

} /* namespace Team302 */

#endif /* ROBOTMAP_H_ */
